--
-- Fendt380GTA
-- Specialization for Fendt 380 GTA
--
-- @author  Manuel Leithner
-- @date  10/01/09
--

Fendt380GTA = {};

function Fendt380GTA.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function Fendt380GTA:load(xmlFile)

	-- Defines attributes of frontHydraulic
	self.frontHydraulic = {};
	self.frontHydraulic.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontHydraulic#hydraulic"));
	self.frontHydraulic.punch = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontHydraulic#hydraulicPunch"));
	self.frontHydraulic.translationPunch = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontHydraulic#hydraulicPunchFixPoint"));
	self.frontHydraulic.armFixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontHydraulic#armFixPoint"));
	local ax, ay, az = getWorldTranslation(self.frontHydraulic.punch);
    local bx, by, bz = getWorldTranslation(self.frontHydraulic.translationPunch);
    self.frontHydraulic.punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);	
	self.frontHydraulic.topArmPart2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.frontHydraulic#part2"));
	
	-- Defines attributes of backHydraulic
	self.backHydraulic = {};
	self.backHydraulic.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#hydraulic"));
	self.backHydraulic.punch = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#hydraulicPunch"));
	self.backHydraulic.translationPunch = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#hydraulicPunchFixPoint"));
	self.backHydraulic.rootArmFixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#rootArmFixPoint"));
	self.backHydraulic.rootArm = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#rootArm"));
	self.backHydraulic.rootArmPart = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#rootArmPart"));
	self.backHydraulic.armFixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#armFixPoint"));
	self.backHydraulic.rootArmPartFixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#rootArmPartFixPoint"));
	self.backHydraulic.balancer = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#balancer"));
	self.backHydraulic.balancerFixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#balancerFixPoint"));
	local ax, ay, az = getWorldTranslation(self.backHydraulic.punch);
    local bx, by, bz = getWorldTranslation(self.backHydraulic.translationPunch);
    self.backHydraulic.punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
	ax, ay, az = getWorldTranslation(self.backHydraulic.rootArmPart);
    bx, by, bz = getWorldTranslation(self.backHydraulic.rootArmPartFixPoint);
    self.backHydraulic.rootArmDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
	
	
	self.backAttacherJoint = {};
	self.backAttacherJoint.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.trailerAttacherJoints#index"));
	self.backAttacherJoint.lock = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.trailerAttacherJoints#lock"));
	self.backAttacherJoint.handGear = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.trailerAttacherJoints#handGear"));
	self.backAttacherJoint.upYPos = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.trailerAttacherJoints#upY"), 0);
	self.backAttacherJoint.downYPos = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.trailerAttacherJoints#downY"), 0);	
	local x,y,z = getTranslation(self.backAttacherJoint.node);
	self.backAttacherJoint.xPos = x;
	self.backAttacherJoint.zPos = z;
	
	
	self.windowsCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.windows#count"), 0);	
	if self.windowsCount == 0 then
        print("Error: No windows specified - Count: 0", configFile);
    end;	
	self.windows = {};	
	for i=1, self.windowsCount do
	
		local windowName = string.format("vehicle.windows.window%d", i);		
		self.windows[i] = {};	
		self.windows[i].node = Utils.indexToObject(self.components, getXMLString(xmlFile, windowName .. "#index"));
		local xRot = Utils.degToRad(Utils.getNoNil(getXMLFloat(xmlFile, windowName .. "#maxRot"), 0));
		self.windows[i].maxRot = {xRot,0,0};
		local x,y,z = getRotation(self.windows[i].node);
		self.windows[i].minRot = {x,y,z}; 
		self.windows[i].movingTime = Utils.getNoNil(getXMLInt(xmlFile, windowName .. "#time"), 2) * 1000;

		local hydraulic = {};
		hydraulic.node = Utils.indexToObject(self.components, getXMLString(xmlFile, windowName.. "#hydraulic"));
		hydraulic.punch = Utils.indexToObject(self.components, getXMLString(xmlFile, windowName .. "#hydraulicPunch"));
		hydraulic.translationPunch = Utils.indexToObject(self.components, getXMLString(xmlFile, windowName .. "#hydraulicPunchFixPoint"));
		hydraulic.windowFixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, windowName .. "#windowFixPoint"));
		local ax, ay, az = getWorldTranslation(hydraulic.punch);
	    local bx, by, bz = getWorldTranslation(hydraulic.translationPunch);
	    hydraulic.punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);	
		hydraulic.scale = Utils.getNoNil(getXMLInt(xmlFile, windowName .. "#scale"), 1);
		self.windows[i].hydraulic = hydraulic;
		
	end;	
	
	-- Defines attributes of drivingShaft
	self.drivingShaft = {};
	self.drivingShaft.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drivingShaft#index"));
	self.drivingShaft.clutch1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drivingShaft#clutch1"));
	self.drivingShaft.clutch2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drivingShaft#clutch2"));
	self.drivingShaft.frontAxisFixpoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drivingShaft#frontAxisFixpoint"));
	self.drivingShaft.motorFixpoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drivingShaft#motorFixpoint"));
	
	
	
	self.isWindowOpen = false;
	self.updateJoint = false;	

end;

function Fendt380GTA:delete()
end;

function Fendt380GTA:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Fendt380GTA:keyEvent(unicode, sym, modifier, isDown)
end;

function Fendt380GTA:update(dt)

	if self.isActive then
	
		if InputBinding.hasEvent(InputBinding.OpenWindows) then
			self.isWindowOpen = not self.isWindowOpen;
		end;
		
		
		if self.windows ~= nil then
			for i=1, table.getn(self.windows) do
				local x, y, z = getRotation(self.windows[i].node);
				local rot = {x,y,z};
				local newRot = Utils.getMovedLimitedValues(rot, self.windows[i].maxRot, self.windows[i].minRot , 3, self.windows[i].movingTime, dt, not self.isWindowOpen);
				local ax, ay, az = unpack(newRot);
				
				if ax ~= x then
					if self.windows[i].hydraulic ~= nil then
						local hydraulic = self.windows[i].hydraulic;					
						local ax, ay, az = getWorldTranslation(hydraulic.node);
						local bx, by, bz = getWorldTranslation(hydraulic.windowFixPoint);
						local x, y, z = worldDirectionToLocal(getParent(hydraulic.node), bx-ax, by-ay, bz-az);							
						local scale = hydraulic.scale;						
						setDirection(hydraulic.node, x*scale, y*scale, z*scale, 0, 1, 1);
						if hydraulic.punch ~= nil then
							local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
							setTranslation(hydraulic.punch, 0, 0, (distance-hydraulic.punchDistance)*scale);
						end;
					end;
				end;
				setRotation(self.windows[i].node, unpack(newRot));
			end;
		end;
		
		-- oriented the hydraulic and the punch towards a fixpoint on the frontHydraulic
		if self.frontHydraulic ~= nil then
			local ax, ay, az = getWorldTranslation(self.frontHydraulic.node);
			local bx, by, bz = getWorldTranslation(self.frontHydraulic.armFixPoint);
			local x, y, z = worldDirectionToLocal(getParent(self.frontHydraulic.node), bx-ax, by-ay, bz-az);
			setDirection(self.frontHydraulic.node, x, y, z, 1, 0, 0);
			if self.frontHydraulic.punch ~= nil then
				local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
				setTranslation(self.frontHydraulic.punch, 0, 0, distance-self.frontHydraulic.punchDistance);
			end;
		end;
		
		if self.drivingShaft ~= nil then
			
			local x,y,z = getRotation(self.wheels[1].driveNode);
			setRotation(self.drivingShaft.node, 0 ,0, x*2);			
			local ax, ay, az = getWorldTranslation(self.drivingShaft.clutch1);
			local bx, by, bz = getWorldTranslation(self.drivingShaft.frontAxisFixpoint);
			x, y, z = worldDirectionToLocal(getParent(self.drivingShaft.clutch1), bx-ax, by-ay, bz-az);
			setDirection(self.drivingShaft.clutch1, x, y, z, 1, 0, 0);	
			ax, ay, az = getWorldTranslation(self.drivingShaft.clutch2);
			bx, by, bz = getWorldTranslation(self.drivingShaft.motorFixpoint);
			x, y, z = worldDirectionToLocal(getParent(self.drivingShaft.clutch2), bx-ax, by-ay, bz-az);
			setDirection(self.drivingShaft.clutch2, x, y, z, 1, 0, 0);		
		end;
		
		
		-- oriented all hydraulicparts towards the right place
		if self.backHydraulic ~= nil then
			-- set correct rotation on rootArm
			local x, y, z = getRotation(getParent(getParent(self.backHydraulic.armFixPoint)));
			setRotation(self.backHydraulic.rootArm, x, y, z);	

			-- oriented rootArmPart towards a fixpoint on the bottomArm
			local ax, ay, az = getWorldTranslation(self.backHydraulic.rootArmPart);
			local bx, by, bz = getWorldTranslation(self.backHydraulic.armFixPoint);
			x, y, z = worldDirectionToLocal(getParent(self.backHydraulic.rootArmPart), bx-ax, by-ay, bz-az);
			setDirection(self.backHydraulic.rootArmPart, x, y, z, 0, 0, 1);	
			
			
			-- scale the rootArmPart to get a better looking style (Do Not Copy This -> WORST implementation but "Yeah it works" ) :-)
			if self.backHydraulic.rootArmPart ~= nil then
				local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
				local xScale, yScale, zScale = getScale(self.backHydraulic.rootArmPart);
				local newScale = zScale * (distance / self.backHydraulic.rootArmDistance);
				setScale(self.backHydraulic.rootArmPart, 1, 1, newScale);
			end;
			
			
			-- oriented balancer towards a fixpoint on the bottomArm
			ax, ay, az = getWorldTranslation(self.backHydraulic.balancer);
			bx, by, bz = getWorldTranslation(self.backHydraulic.balancerFixPoint);
			x, y, z = worldDirectionToLocal(getParent(self.backHydraulic.balancer), bx-ax, by-ay, bz-az);
			setDirection(self.backHydraulic.balancer, x*-1, y*-1, z*-1, 0, 1, 0);
			
		
			-- oriented hydraulic towards a fixpoint on the rootArm
			ax, ay, az = getWorldTranslation(self.backHydraulic.node);
			bx, by, bz = getWorldTranslation(self.backHydraulic.rootArmFixPoint);
			x, y, z = worldDirectionToLocal(getParent(self.backHydraulic.node), bx-ax, by-ay, bz-az);
			setDirection(self.backHydraulic.node, x, y, z, 0, 1, 1);
			if self.backHydraulic.punch ~= nil then
				local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
				setTranslation(self.backHydraulic.punch, 0, 0, (distance-self.backHydraulic.punchDistance));
			end;			
		end;	
	end;
end;

function Fendt380GTA:draw()

	if self.isActive then
		if self.isWindowOpen then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("close_window"), self.typeDesc), InputBinding.OpenWindows);
		else
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("open_window"), self.typeDesc), InputBinding.OpenWindows);
		end;
	end;
	
end;

function Fendt380GTA:attachImplement(implement)
	
	local jointType = implement.object.attacherJoint.jointType;
	local jointIndex = implement.jointDescIndex;
		
	if jointType == Vehicle.JOINTTYPE_IMPLEMENT then
	
		-- index 1 defines back hydraulic
		if jointIndex == 1 then
			setVisibility(self.backAttacherJoint.node, false);
		end;
		
	elseif jointType == Vehicle.JOINTTYPE_TRAILER then
	
		setVisibility(self.backAttacherJoint.node, true);	
		setTranslation(self.backAttacherJoint.node, self.backAttacherJoint.xPos, self.backAttacherJoint.upYPos, self.backAttacherJoint.zPos);
		
	elseif jointType == Vehicle.JOINTTYPE_TRAILERLOW then
	
		setVisibility(self.backAttacherJoint.node, true);	
		setTranslation(self.backAttacherJoint.node, self.backAttacherJoint.xPos, self.backAttacherJoint.downYPos, self.backAttacherJoint.zPos);
	end;
	self.updateJoint = true;
	
end;

function Fendt380GTA:validateAttacherJoint(implement, jointDesc, dt)
    if self.updateJoint then
        self.updateJoint = false;
        return true;
    end;
    return false;
end;


